'use strict';
const Schema = require('mongoose').Schema;
const moment = require('moment');
const metaPlugin = require('naf-framework-mongoose-free/lib/model/meta-plugin');
const { Secret } = require('naf-framework-mongoose-free/lib/model/schema');

// 机构表
const organization = {
  name: { type: String, required: true, maxLength: 200 }, // 用户名
  phone: { type: String, required: false, maxLength: 200 }, // 电话号码
  password: { type: Secret, required: true, select: false }, // 登录密码
  code: { type: String, required: false, default: "INVESTORS" }, // 邀请码
  role: { type: String, required: false, default: "4" }, // 角色
  email: { type: String, required: false, maxLength: 200 }, // 邮箱
  addr: { type: String, required: false, maxLength: 500 }, // 地址
  office_phone: { type: String, required: false, maxLength: 500 }, // 办公电话
  profession: { type: String, required: false, maxLength: 500 }, // 所属行业
  juris: { type: String }, // 辖区

  institution_code: { type: String, required: false, maxLength: 500 }, // 机构代码
  companytype: { type: String, required: false, maxLength: 300 }, // 注册类型
  companydate: { type: String, required: false, maxLength: 300 }, // 注册时间
  companycapital: { type: String, required: false, maxLength: 300 }, // 注册资金
  companyperson: { type: String, required: false, maxLength: 300 }, // 企业法人
  sndqyzsr: { type: String, required: false, maxLength: 300 }, // 上年度企业总收入
  sndyffy: { type: String, required: false, maxLength: 300 }, // 上年度研发费用
  companytotal: { type: String, required: false, maxLength: 300 }, // 企业总人数
  zjzyfrs: { type: String, required: false, maxLength: 300 }, // 专&兼职研发人数
  companybrief: { type: String, required: false }, // 企业简介
  mainproduct: { type: String, required: false, maxLength: 300 }, // 主要产品
  qualifications: { type: String, required: false, maxLength: 300 }, // 企业资质&荣誉

  status: { type: String, required: false, default: "0", maxLength: 200 }, // 审核状态,0-注册,1-通过,2-拒绝
  openid: { type: String }, // 微信openid
  isdel: { type: String, required: false, default: "0" }, // 0=>未删除;1=>已删除
  remark: { type: String, maxLength: 200 },
  create_time: { type: String },
};
const schema = new Schema(organization, { toJSON: { virtuals: true } });
schema.index({ id: 1 });
schema.index({ phone: 1 });
schema.index({ profession: 1 });
schema.index({ institution_code: 1 });
schema.index({ companyperson: 1 });
schema.index({ 'meta.createdAt': 1 });
schema.plugin(metaPlugin);
module.exports = app => {
  const { mongoose } = app;
  return mongoose.model('Organization', schema, 'organization');
};
