'use strict';
const { AxiosService } = require('naf-framework-mongoose-free/lib/service');
const { BusinessError, ErrorCode } = require('naf-core').Error;
const { isNullOrUndefined } = require('naf-core').Util;
const _ = require('lodash');

//
class HttpUtilService extends AxiosService {
  constructor(ctx) {
    super(ctx, {}, {});
  }


  // 替换uri中的参数变量
  merge(uri, query = {}) {
    const keys = Object.keys(query);
    const arr = [];
    for (const k of keys) {
      arr.push(`${k}=${query[k]}`);
    }
    if (arr.length > 0) {
      uri = `${uri}?${arr.join('&')}`;
    }
    return uri;
  }

  /**
   * curl-get请求
   * @param {String} uri 接口地址
   * @param {String} project config中的项目key
   * @param {Object} query 地址栏参数
   * @param {Object} options 额外参数
   */
  async cget(uri, project, query, options) {
    return this.toRequest(uri, project, null, query, options);
  }

  /**
   * curl-post请求
   * @param {String} uri 接口地址
   * @param {String} project config中的项目key
   * @param {Object} data post的body
   * @param {Object} query 地址栏参数
   * @param {Object} options 额外参数
   */
  async cpost(uri, project, data = {}, query, options) {
    return this.toRequest(uri, project, data, query, options);
  }

  async toRequest(uri, project, data, query, options) {
    const prefix = _.get(this.ctx.app.config.project, project);
    if (!prefix) {
      throw new BusinessError(
        ErrorCode.SERVICE_FAULT,
        `未设置用户权限项目的关联:config.project.${project} is undefined`
      );
    }
    query = _.pickBy(
      query,
      val => val !== '' && val !== 'undefined' && val !== 'null'
    );
    if (!uri) console.error('uri不能为空');
    if (_.isObject(query) && _.isObject(options)) {
      const params = query.params ? query.params : query;
      options = { ...options, params };
    } else if (_.isObject(query) && !query.params) {
      options = { params: query };
    } else if (_.isObject(query) && query.params) {
      options = query;
    }
    // 是否多租户模式,需要改变headers
    const headers = { 'content-type': 'application/json' };
    const url = this.merge(`${prefix}${uri}`, options.params);
    let res = await this.ctx.curl(url, {
      method: isNullOrUndefined(data) ? 'get' : 'post',
      url,
      data,
      dataType: 'json',
      headers,
      ...options,
    });
    if (res.status === 200) {
      res = res.data || {};
      const { errcode, errmsg, details } = res;
      if (errcode) {
        console.warn(`[${uri}] fail: ${errcode}-${errmsg} ${details}`);
        return { errcode, errmsg };
      }
      return res.data;
    }
    const { status } = res;
    console.warn(`[${uri}] fail: ${status}-${res.data.message} `);
  }
}

module.exports = HttpUtilService;
